;________________________________________________________________________
;
;                             A P R S P E E D . A S M
;________________________________________________________________________
;
; APRS Position Sending using TAPR's PIC-E with adaptative beacon time
; by Colin Buckup N5GG April 00 - please e-mail to n5gg@hotmail.com
; based on the original code PICE90 for Position Sending with the PIC-E
; previously written by Byon Garrabrant  N6BG  byon@mail.com  Feb 99
; All copyrights that apply do Byon's code also apply to this version
;________________________________________________________________________
;
; Revision
; v 1.0 - 04/06/00 - Colin
;
;________________________________________________________________________
; VERY IMPORTANT:
; Please remember to configure SETTINGS.ASM for your own station
; parameters before compiling.
;________________________________________________________________________
;
; Description:
; This software is based on the original PICE90 software written by N6BG.
;
; The main change is that the beacon period is now variable according to
; the current speed of the mobile. This means that the rate of packets sent
; will change based on the current conditions of the mobile.
;
; The speed x position update period table is as follows:
; Speed range   Beacon period
; 0 - 1 kn      15 minutes
; 2 - 4 kn      5 minutes
; 5 - 11 kn     2 minutes
; 12 - 21 kn    1 minute
; 22 - 38 kn    30 seconds
; 39 - 67 kn    20 seconds
; 68 and above  10 seconds
;
; With the above values, it's ensured that the position of the mobile will
; be updated in a rate such that the mobile's distance traveled during the
; period between position updates is less than a 1/4 mile (worst case
; is about 700 meters).
; It will also lessen the number of position packets sent out to only one
; every 15 minutes if the mobile is not moving, therefore not clogging the
; frequency with redundant and useless information.
; The software has a Max_Speed memory for the current update period, so
; that the time between packets is always based on the Max Speed during
; that period. This means that if the mobile was stationary and starts
; moving, it will immediately update it's beacon rate to the new conditions.
; I hope this version of the code improves the "position accuracy" and
; the band usage.
; 73 de Colin N5GG / ex AC5TV, ex PY2CGB - Lewisville, TX - April 2000.
;________________________________________________________________________
;
	LIST	P=PIC16F84
	include	"P16F84.INC"
	__CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF
;________________________________________________________________________

	cblock	H'0C'			; system variables
		beacon_counter
		temp
		temp2
	endc
;________________________________________________________________________

#define	PTT_OUT		PORTA, 2
;________________________________________________________________________

	ORG	0			; start code
	goto	main
;________________________________________________________________________
; I changed the name of all the ASM files to avoid confusion with the 
; PICE90 original code package. Colin.

	include	"SETTINGS.ASM"	; include options code
	include	"MODEM.ASM"		; include MX614 Modem code
	include	"GPS.ASM"		; include NMEA code
	include	"PACKET.ASM"	; include AX.25 UI code
	include	"SERDATA.ASM"	; include serial code
	include	"TIMEBASE.ASM"	; include timing code
;________________________________________________________________________
; Various packet routines

AX25_Send_Space
	movlw	' '

AX25_Send_Byte
	goto	AX25_Send_Data

AX25_Send_Hex
	movwf	temp
	swapf	temp, W
	call	AX25_Send_Hex_Nibble
	movfw	temp

AX25_Send_Hex_Nibble
	andlw	H'0F'
	sublw	9
	skpc
	 addlw	-7
	sublw	'0'+9
	goto	AX25_Send_Byte
;________________________________________________________________________

Packet_Send
	bsf	PTT_OUT			; press PTT
	call	AX25_Send_Start

	movlw	MIC_E_DEST_BASE 	; find the current offset
	movwf	FSR		

Packet_Send_Dest_Loop
	movfw	INDF
	call	AX25_Send_Data

	incf	FSR, F
	movfw	FSR			; check if we have receive
	xorlw	MIC_E_DEST_BASE + MIC_E_DEST_LEN ; all chars of the string
	skpz				; if not
	 goto	Packet_Send_Dest_Loop

	movlw	OPT_CALLSIGN_DIGI
	movwf	temp

Packet_Send_Callsign
	movfw	temp
	call	Get_Option
	iorlw	0
	skpnz
	 goto	Packet_Send_Type

	movwf	temp2
	incf	temp, W
	call	Get_Option
	iorlw	0
	clrc
	skpnz
	 setc
 	rlf	temp2, W

	call	AX25_Send_Data
	incf	temp, F
	goto	Packet_Send_Callsign

Packet_Send_Type
	movlw	H'03'
	call	AX25_Send_Data
	movlw	H'F0'
	call	AX25_Send_Data

Packet_Send_Info_Loop
	movfw	INDF
	call	AX25_Send_Data

	incf	FSR, F
	movfw	FSR			; check if we have receive
	xorlw	MIC_E_INF_BASE + MIC_E_INF_LEN	; all chars of the string
	skpz				; if not
	 goto	Packet_Send_Info_Loop

	movlw	OPT_SYMBOL		; get and send the symbol
	call	Get_Option
	call	AX25_Send_Data

	movlw	OPT_SYMBOL_TABLE	; get and send the symbol table
	call	Get_Option
	call	AX25_Send_Data

	decfsz	beacon_counter, F
	 goto	Packet_Beacon_Done

	movlw	OPT_BEACON_RATE	; get the beacon rate
	call	Get_Option
	movwf	beacon_counter

	movlw	OPT_BEACON_OFFSET
	call	Get_Option
	movwf	temp

Packet_Send_Beacon
	movfw	temp
	call	Get_Option
	iorlw	0
	skpnz
	 goto	Packet_Beacon_Done
	call	AX25_Send_Data
	incf	temp, F
	goto	Packet_Send_Beacon

Packet_Beacon_Done

	call	AX25_Send_End
	bcf	PTT_OUT			; release PTT
	return
;________________________________________________________________________
; Here is where the main program starts

main
	call	serialInit
	call	NMEA_Init
	call	Tone_Init
	call	Timing_Init

	bsf	STATUS, RP0		; Select Bank 1
	bcf	PTT_OUT			; make PTT an output
	bcf	VALID_OUT
	bcf	STATUS, RP0		; Select Bank 0
	bcf	PTT_OUT			; release PTT

	movlw	1
	movwf	beacon_counter

mainLoop
	call	serialReceive		; receive serial
	iorlw	0			; look at result
	skpz				; if non zero
	 call	NMEA_Decode		;  send to NMEA decoder

	call	Timing_Check		; check timing and inputs
	iorlw	0			; look at result
	skpz				; if non zero
	 call	Packet_Send		; send a packet

	goto	mainLoop		; loop back again

	END
;________________________________________________________________________
; GPS sentences:
;type   time   A lat        long        spd   bear  date   dev     cs
;$GPRMC,083036,A,3610.999,N,11516.337,W,219.6,326.6,221197,013.7,E*6F
;$GPRMC,072004,A,5124.164,N,00039.253,W,021.7,028.9,180496,004.1,W*78 
;$GPRMC,000000,A,3046.993,N,08140.503,W,100.0,100.0,090798,000.0,W*78 
